#!/usr/bin/env python3

import sys
import praw
import warnings
import argparse
import os
from yaml import load
from pyprind import ProgBar
import mistune
import uuid
from .ebooklib_patched import epub
from PIL import Image
from pkg_resources import resource_string


def main():
    parser = argparse.ArgumentParser(
        description="Create epubs from Reddit")
    parser.add_argument('config_file',
                        help="The file containing the links, formatted in YAML")
    parser.add_argument('output_directory')
    args = parser.parse_args()

    config_file = args.config_file
    directory = args.output_directory

    # Disable warnings due to a bug in praw
    warnings.filterwarnings("ignore")

    reader = praw.Reddit(user_agent="reddit2ebook")
    config = read_config_file(config_file)
    # We have to use xhtml here for Epubs
    renderer = mistune.Renderer(use_xhtml=True, escape=True)
    markdown = mistune.Markdown(renderer=renderer)

    for bookname in config.keys():
        book = epub.EpubBook()
        # The epub standard requires an unique identifier, this is normally
        # the ISBN, but since we dont have one we generate an UUID
        book.set_identifier(uuid.uuid4().hex)

        chapter_number = 1
        chapters = []

        if "links" in config[bookname]:
            bar = ProgBar(
                len(config[bookname]["links"]),
                title="Creating ebook " + bookname + ".epub",
                bar_char='█')

            links = config[bookname]["links"]

            if "cover" in config[bookname]:
                convert_to_jpeg(config[bookname]["cover"])
                with open("cover.jpg", 'rb') as f:
                    book.set_cover("cover.jpg", f.read())
                os.remove("cover.jpg")

            if "author" in config[bookname]:
                book.add_author(config[bookname]["author"])

            if "lang" in config[bookname]:
                book.set_language(config[bookname]["lang"])
            else:
                book.set_language('en')

            if "title" in config[bookname]:
                book.set_title(config[bookname]["title"])
            else:
                book.set_title(bookname)

        else:
            links = config[bookname]
            book.set_language('en')
            book.set_title(bookname)
            bar = ProgBar(len(config[bookname]),
                          title="Creating ebook " + bookname + ".epub",
                          bar_char='█')

        for url in links:
            bar.update()
            # Check if the link is a comment or a submission
            # Submissions have a trailing slash
            if url.split('/')[-1] == '':
                submission = get_submission_text(reader, url)
                chapter = create_chapter(
                    body=markdown(submission[1]),
                    title=submission[0],
                    filename="chapter" + str(chapter_number) + ".xhtml"
                )
            else:
                comment = get_comment_text(reader, url)
                chapter = create_chapter(
                    body=markdown(comment[1]),
                    title="Comment by " + comment[0],
                    filename="chapter" + str(chapter_number) + ".xhtml"
                )

            chapters.append(chapter)

            book.add_item(chapter)

            chapter_number += 1

        book.toc = chapters
        book.add_item(epub.EpubNcx())
        book.add_item(epub.EpubNav())

        style = load_css()
        default_css = epub.EpubItem(
            uid="style_default",
            file_name="style/default.css",
            media_type="text/css",
            content=style)
        book.add_item(default_css)

        nav_css = epub.EpubItem(
            uid="style_nav",
            file_name="style/nav.css",
            media_type="text/css",
            content=style)
        book.add_item(nav_css)

        spine = ['cover', 'nav'] + chapters

        book.spine = spine

        epub.write_epub(os.path.join(directory, bookname + ".epub"), book, {})

        print("Finished writing " + bookname + ".epub\n")


def read_config_file(config_file):
    '''
    Load the config_file using YAML
    :param config_file:
    :return:
    '''
    try:
        with open(config_file, 'r') as file:
            config = load(file)
    except OSError:
        print("Config file not found")
        sys.exit(0)

    return config


def get_comment_text(reader, url):
    submission = reader.get_submission(url)
    comment = submission.comments[0]
    name = comment.author.name
    text = comment.body
    return (name, text)


def get_submission_text(reader, url):
    submission = reader.get_submission(url)
    title = submission.title
    text = submission.selftext
    return (title, text)


def create_chapter(title, body, filename):
    chapter = epub.EpubHtml(title=title, file_name=filename)
    chapter.content = body
    return chapter


def load_css():
    # Currently uses the Epub CSS starter kit as a sane default
    # https://github.com/mattharrison/epub-css-starter-kit/
    style = resource_string(__name__, 'base.css')
    return style


def convert_to_jpeg(file_name):
    im = Image.open(file_name)
    im.save('cover.jpg')

if __name__ == "__main__":
    main()



# This file is part of EbookLib.
# Copyright (c) 2013 Aleksandar Erkalovic <aerkalov@gmail.com>
#
# EbookLib is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EbookLib is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with EbookLib.  If not, see <http://www.gnu.org/licenses/>.

import io
from lxml import etree

def debug(obj):
    import pprint

    pp = pprint.PrettyPrinter(indent=4)
    pp.pprint(obj)

def parse_string(s):
    try:
        tree = etree.parse(io.BytesIO(s.encode('utf-8')))
    except:
        tree = etree.parse(io.BytesIO(s))

    return tree

def parse_html_string(s):
    from lxml import html

    utf8_parser = html.HTMLParser(encoding='utf-8')

    html_tree = html.document_fromstring(s , parser=utf8_parser)

    return html_tree



# This file is part of EbookLib.
# Copyright (c) 2013 Aleksandar Erkalovic <aerkalov@gmail.com>
#
# EbookLib is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# EbookLib is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with EbookLib.  If not, see <http://www.gnu.org/licenses/>.

import six
import subprocess

from ebooklib.plugins.base import BasePlugin
from ebooklib.utils import parse_html_string

# Recommend usage of
# - https://github.com/w3c/tidy-html5

def tidy_cleanup(content, **extra):
    cmd = []

    for k, v in six.iteritems(extra):
    	cmd.append('--%s' % k)

    	if v:
    		cmd.append(v)

    # must parse all other extra arguments
    try:
        p = subprocess.Popen(['tidy']+cmd, shell=False, 
                             stdin=subprocess.PIPE, stdout=subprocess.PIPE, 
                             stderr=subprocess.PIPE, close_fds=True)
    except OSError:
        return (3, None)

    p.stdin.write(content)

    (cont, p_err) = p.communicate()

    # 0 - all ok
    # 1 - there were warnings
    # 2 - there were errors
    # 3 - exception

    return (p.returncode, cont)


class TidyPlugin(BasePlugin):
    NAME = 'Tidy HTML'
    OPTIONS = {'utf8': None,
               'tidy-mark': 'no'
              }

    def __init__(self, extra = {}):
        self.options = dict(self.OPTIONS)
        self.options.update(extra)

    def html_before_write(self, book, chapter):
        if not chapter.content:
            return None

        (_, chapter.content) = tidy_cleanup(chapter.content, **self.options)

        return chapter.content

    def html_after_read(self, book, chapter):
        if not chapter.content:
            return None

        (_, chapter.content) = tidy_cleanup(chapter.content, **self.options)

        return chapter.content




"""Convert reddit stories and comments into ebooks
See:
https://github.com/MissingNoIOI/reddit2ebook
"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

    setup(
        name='reddit2ebook',

        version='2.0.0',

        description='convert reddit stories and comments into ebooks',
        long_description=long_description,

        url='https://github.com/MissingNoIOI/reddit2ebook',

        author='Gerrit Helling',
        author_email='gerrit@akuma.pictures',

        license='GPLv3',

        include_package_data=True,

        classifiers=[

            'Development Status :: 4 - Beta',

            'Environment :: Console',

            'Intended Audience :: End Users/Desktop',
            'Topic :: Multimedia',

            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

            'Operating System :: OS Independent',

            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5'
        ],

        keywords='reddit ebook epub offline',

        packages=find_packages(),

        install_requires=[
            'praw',
            'PyYaml',
            'pypub',
            'mistune',
            'pyprind',
            'Pillow'
        ],

        entry_points={
            'console_scripts': [
                'reddit2ebook=reddit2ebook.main:main',
            ],
        },

    )



"""
CLI Program to turn reddit submissions and comments into Ebooks

 Usage: reddit2ebook initfile directory

 The initfile should be in the following format:

 bookname:
     - url
     - url
     - url

"""
